/********************************************************************************
						Graffitti Board Code
					  Copyright Sparfun Electronics
					    Written by Ryan Owens
						       9/4/2008
*********************************************************************************
*/

//***************************************
//			Include Libraries
//***************************************
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

//***************************************
//		Macros and Definitions
//***************************************
#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

//LED Output Pins (PortB)
#define LED1	0
#define LED2	1
#define LED3	3

//LED Macros
#define LED1_ON()	sbi(PORTB, LED1)
#define LED1_OFF()	cbi(PORTB, LED1)

#define LED2_ON()	sbi(PORTB, LED2)
#define LED2_OFF()	cbi(PORTB, LED2)

#define LED3_ON()	sbi(PORTB, LED3)
#define LED3_OFF()	cbi(PORTB, LED3)

//Infrared Emitter/Detector
#define SENSE	2
#define ENABLE	4

//Program Definitions
#define SAMPLES	16
#define INT_FREQ 60000
#define FRAME_DEPTH 511
#define DIM_LEVELS	50
#define DIM_CONSTANT	INT_FREQ/DIM_LEVELS

//***************************************
//			Global Variables
//***************************************
volatile int led_brightness;
volatile unsigned long int light_up=0;
volatile int frame_count=0;

//***************************************
//		Function Prototypes
//***************************************
void ioinit (void);      // initializes IO
void delay_ms(uint16_t x);

//***************************************
//			Main Code
//***************************************
ISR(SIG_OUTPUT_COMPARE0A){
	cli();
	if(frame_count < FRAME_DEPTH)frame_count+=1;
	else frame_count=0;
	
	if(light_up>0){
		light_up-=1;
		if(led_brightness > frame_count){
			LED1_ON();
			LED2_ON();
			LED3_ON();
		}
		else{
			LED1_OFF();
			LED2_OFF();
			LED3_OFF();
		}
	}
	else{
		LED1_OFF();
		LED2_OFF();
		LED3_OFF();
	}
	sei();
}

int main (void)
{
	int infrared_reading=0, last_reading=200;
	ioinit();
	
	
	while(1){
		sbi(PORTB, ENABLE);		//Turn on the Emitter
		for(int i = 0; i < SAMPLES; i++)		//Read ADC Port
		{
			int temp;
			ADCSRA |= (1<<ADSC);
			
			//Conversion starts - wait for ADSC to clear
			while( (ADCSRA & (1<<ADSC)) != 0 );
			
			temp = ADCL;
			temp |= ADCH<<8;
			
			infrared_reading += temp;
			
		}	
		cbi(PORTB, ENABLE);	//Turn off Emitter to conserve power
		infrared_reading/=SAMPLES;	//Find out what the average reading was.
		
		//Find out if the IR detector noticed a change in the reading
		if(infrared_reading > last_reading+25){	//If the IR reading changed significantly, the light up!
			light_up=INT_FREQ;			//We're going to light up for 1 second.
			//led_brightness=9;
			led_brightness=500;
		}
		last_reading=infrared_reading;
		infrared_reading=0;
		
		//!!!//
		//Wait to finish the light sequence before reading the accelerometer again.
		//If I remove this line, the LED sequence should be changed in "mid-sequence" if IR is detected again.
		//while(light_up>0)led_brightness=light_up/(DIM_CONSTANT);
		while(light_up>0);
	}
	
return 0;	
}

void ioinit (void) 
{

	//Set up the I/O
    //1 = output, 0 = input
	DDRB |= (1<<LED1) | (1<<LED2) | (1<<LED3); //Enable the LED pins as outputs
	DDRB |= (1<<ENABLE);	//Set the emitter enable line as an output
	DDRB &= ~(1<<SENSE);	//Set the ADC pin as an input for reading the detector pin
	//PORTB |= (1<<SENSE);	//Enable the pull-up resistor...for now...
	
	//Set up the A/D converter
	ADMUX = (1<<MUX0);	//Select channel 1 (PB2) for the ADC channel
	ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); //Turn on the ADC, and divide the system clock by 128 for conversions

	TCCR0A=(1<<WGM01);	//Clear timer on Compare
	TCCR0B=(1<<CS01); //Divide Clock(4.8MHZ) by 8 for 600KHz Timer
	OCR0A=20;	//Set Interrupt frequency to Timer(600KHz)/20=30kHz
	TIMSK0 = (1<<OCIE0A);		//Interrupt when timer0=OCA
	
	sei(); //Enable interrupts	//Enable Interrupts
}

//General short delays
void delay_ms(uint16_t x)
{
  uint8_t y, z;
  for ( ; x > 0 ; x--){
    for ( y = 0 ; y < 40 ; y++){
      for ( z = 0 ; z < 40 ; z++){
        asm volatile ("nop");
      }
    }
  }
}
